
  /***********************************************************************/
  /****   Copyright (C) 1991, 1992, 1993, 1994, 1995 by Brian Reed.   ****/
  /****   All Rights Reserved.  Written by Brian Reed.                ****/
  /***********************************************************************/
  /****   The prototype file for the BF library.  Called by BF.H.     ****/
  /***********************************************************************/


/********  Protect against multiple inclusion.  ********/
#ifndef BFPROTO_H
#define BFPROTO_H


  /********  Tell CPP that we're C functions.  ********/

  #ifdef __cplusplus
    extern "C" {
  #endif



  /********  Get the current display mode.  ********/
  #define bfGetMode()  (bfGlobals.font_current_mode)


  /********  Here are the file utility definitions and protos.  ********/
  /*  These are the only tested file modes and seek types.  */
  #define FILE_READONLY    0
  #define FILE_READWRITE   2
  #define FILE_SEEKSET     0
  #define FILE_SEEKCUR     1
  #define FILE_SEEKEND     2
  extern SWORD bfDECL dActCreate(SBYTE bfFAR *name);
  extern SWORD bfDECL dActOpen(SBYTE bfFAR *name, UWORD oflags);
  extern void  bfDECL dActClose(SWORD handle);
  extern UWORD bfDECL dActRead(SWORD handle, SBYTE bfFAR *data, UWORD cnt);
  extern UWORD bfDECL dActWrite(SWORD handle, SBYTE bfFAR *data, UWORD cnt);
  extern SLONG bfDECL dActSeek(SWORD handle, SLONG offs, SWORD fromwhere);



  /*  HIGH-LEVEL functions.  */

  /********  Initialize the font system.  ********/
  extern SWORD  bfDECL bfActInit(bfMode initmode);

  /********  End the font system usage.  ********/
  extern SWORD  bfDECL bfActEnd(void);

  /********  Open a font file.  ********/
  extern bfFont bfFAR * bfDECL bfActOpen(SBYTE bfFAR *bfname);

  /********  Open a TrueType font file.  ********/
  #ifdef bfWIN
    extern bfFont bfFAR * bfDECL bfActOpenTT(HDC hdc,
                                             LPLOGFONT lplf,
                                             SBYTE bfFAR *chars,
                                             bfCompress compression);
  #endif

  /********  Close an opened font.  ********/
  extern SWORD  bfDECL bfActClose(bfFont bfFAR *font);

  /********  Close all fonts.   ********/
  extern SWORD bfDECL bfActCloseAll(void);

  /********  Make a font state in memory.  ********/
  extern bfState bfFAR * bfDECL bfActStateMake(void);

  /********  Destroy font State.  ********/
  extern bfState bfFAR * bfDECL bfActStateDestroy(bfState bfFAR *state);

  /********  Get font image key information.  ********/
  extern bfState bfFAR * bfDECL bfGetImage(bfState bfFAR *state,
                                           bfBitMapPtr bfFAR *bm,
                                           UWORD bfFAR *iposx,
                                           UWORD bfFAR *iposy);

  /********  Set font image key information.  ********/
  extern bfState bfFAR * bfDECL bfSetImage(bfState bfFAR *state,
                                           bfBitMapPtr bm,
                                           UWORD iposx,
                                           UWORD iposy);

  /********  Set current destination BitMap.  ********/
  extern bfState bfFAR * bfDECL bfSetBitMap(bfState bfFAR *state,
                                            bfBitMapPtr bm);

  /********  Get current destination BitMap.  ********/
  extern bfBitMapPtr bfDECL bfGetBitMap(bfState bfFAR *state);


  /*  Draw functions.  */

  /********  Append a character after the last one drawn.  ********/
  extern bfState bfFAR * bfDECL bfChrAppend(bfState bfFAR *state,
                                            SWORD chr,
                                            SWORD nextchr);

  /********  Draw a character in a font.  ********/
  extern bfState bfFAR * bfDECL bfChrDraw(bfState bfFAR *state,
                                          SWORD x,
                                          SWORD y,
                                          SWORD chr,
                                          SWORD nextchr);

  /********  Append a string after the last char drawn.  ********/
  extern SBYTE bfFAR * bfDECL bfStrAppend(bfState bfFAR *state,
                                          SBYTE bfFAR *str);

  /********  Append a string (N chrs max) after the last drawn.  ********/
  extern SBYTE bfFAR * bfDECL bfStrAppendN(bfState bfFAR *state,
                                           SWORD nchars,
                                           SBYTE bfFAR *str);

  /********  Draw a font text string.  ********/
  extern SBYTE bfFAR * bfDECL bfStrDraw(bfState bfFAR *state,
                                        SWORD x,
                                        SWORD y,
                                        SBYTE bfFAR *str);

  /********  Draw a font text string (N chars max).  ********/
  extern SBYTE bfFAR * bfDECL bfStrDrawN(bfState bfFAR *state,
                                         SWORD x,
                                         SWORD y,
                                         SWORD nchars,
                                         SBYTE bfFAR *str);

  #if 0
  /********  Get a string from the keyboard using a font.    ********/
  extern SWORD bfDECL bfStrInput(bfState bfFAR *state,
                                 bfFont *font,
                                 SWORD x,
                                 SWORD y,
                                 bfBackMode backmode,
                                 SBYTE *str,
                                 SWORD maxlen,
  #endif

  /********  Display a string, wrapping text.  ********/
  extern SBYTE bfFAR * bfDECL bfStrWrap(bfState bfFAR *state,
                                        SWORD x,
                                        SWORD y,
                                        SBYTE bfFAR *str,
                                        SWORD line_width,
                                        SWORD line_spacing);


  /*  MODE and COLOR functions.  */

  /********  Get font draw mode.  ********/
  extern bfState bfFAR * bfDECL bfGetDrawMode(bfState bfFAR *state,
                                              bfDrawMode bfFAR *drawmode,
                                              UWORD bfFAR *ratio);

  /********  Set font draw mode.  ********/
  extern bfState bfFAR * bfDECL bfSetDrawMode(bfState bfFAR *state,
                                              bfDrawMode drawmode,
                                              UWORD ratio);

  /********  Get the drop shadow drawmode.  ********/
  extern bfState bfFAR * bfDECL bfGetDropMode(bfState bfFAR *state,
                                              bfDrawMode bfFAR *drawmode,
                                              UWORD bfFAR *ratio);

  /********  Set the drop shadow drawmode.  ********/
  extern bfState bfFAR * bfDECL bfSetDropMode(bfState bfFAR *state,
                                              bfDrawMode drawmode,
                                              UWORD ratio);

  /********  Get font background mode.  ********/
  extern bfState bfFAR * bfDECL bfGetBackMode(bfState bfFAR *state,
                                              bfDrawMode bfFAR *mode,
                                              UWORD bfFAR *ratio);

  /********  Set font background mode.  ********/
  extern bfState bfFAR * bfDECL bfSetBackMode(bfState bfFAR *state,
                                              bfDrawMode drawmode,
                                              UWORD ratio);

  /********  Get the drop shadow distance.  ********/
  extern bfState bfFAR * bfDECL bfGetDropDist(bfState bfFAR *state,
                                              SWORD bfFAR *dropdistx,
                                              SWORD bfFAR *dropdisty);

  /********  Set the drop shadow distance.  ********/
  extern bfState bfFAR * bfDECL bfSetDropDist(bfState bfFAR *state,
                                              SWORD dropdistx,
                                              SWORD dropdisty);

//  /********  Get font background drop draw mode.  ********/
//  extern bfState bfFAR * bfDECL bfGetBackDrop(bfState bfFAR *state,
//                                              bfColor bfFAR *fg,
//                                              SWORD bfFAR *fg256,
//                                              bfDrawMode bfFAR *mode,
//                                              UWORD bfFAR *ratio);
//
//  /********  Set font background drop mode.  ********/
//  extern bfState bfFAR * bfDECL bfSetBackDrop(bfState bfFAR *state,
//                                              bfColor fg,
//                                              SWORD fg256,
//                                              bfDrawMode drawmode,
//                                              UWORD ratio);

  /********  Get background style.  ********/
  extern bfState bfFAR * bfDECL bfGetBackStyle(bfState bfFAR *state,
                                               bfBackStyle bfFAR *style,
                                               SWORD bfFAR *stylesize);

  /********  Set background style.  ********/
  extern bfState bfFAR * bfDECL bfSetBackStyle(bfState bfFAR *state,
                                               bfBackStyle style,
                                               SWORD stylesize);

  /********  Get background 'added' size.  ********/
  extern bfState bfFAR * bfDECL bfGetBackSize(bfState bfFAR *state,
                                              SWORD bfFAR *addedL,
                                              SWORD bfFAR *addedT,
                                              SWORD bfFAR *addedR,
                                              SWORD bfFAR *addedB);

  /********  Set background 'added' size.  ********/
  extern bfState bfFAR * bfDECL bfSetBackSize(bfState bfFAR *state,
                                              SWORD addedL,
                                              SWORD addedT,
                                              SWORD addedR,
                                              SWORD addedB);

  /********  Get the color settings.  ********/
  extern bfState bfFAR * bfDECL bfGetColors(bfState bfFAR *state,
                                            SWORD bfFAR *fore256,
                                            SWORD bfFAR *back256,
                                            bfColor bfFAR *fg,
                                            bfColor bfFAR *bg);

  /********  Set the color.  ********/
  extern bfState bfFAR * bfDECL bfSetColors(bfState bfFAR *state,
                                            SWORD fg256,
                                            SWORD bg256,
                                            bfColor fg,
                                            bfColor bg,
                                            SWORD gradate_now,
                                            SWORD beta,
                                            REAL gamma);

  /********  Get the drop shadow distance and colors.  ********/
  extern bfState bfFAR * bfDECL bfGetDropColors(bfState bfFAR *state,
                                                SWORD bfFAR *drop256,
                                                bfColor bfFAR *dropclr);

  /********  Set the drop shadow distance and colors.  ********/
  extern bfState bfFAR * bfDECL bfSetDropColors(bfState bfFAR *state,
                                                SWORD drop256,
                                                bfColor dropclr,
                                                bfColor dropbg,
                                                SWORD gradate_now,
                                                SWORD beta,
                                                REAL gamma);


  /*  MISC functions.   */

  /********  Get range of defined chars in font.  ********/
  extern bfFont bfFAR * bfDECL bfGetChrRange(bfFont bfFAR *font,
                                             SWORD bfFAR *ch_min,
                                             SWORD bfFAR *ch_max);

  /********  Get the width of a character.  ********/
  extern SWORD bfDECL bfGetChrWidth(bfFont bfFAR *font,
                                    SWORD ch);

  /********  Get the height of a char.  ********/
  extern SWORD bfDECL bfGetChrHeight(bfFont bfFAR *font,
                                      int ch);

  /********  Get the ascent distance of a char.  ********/
  extern SWORD bfDECL bfGetChrAscent(bfFont bfFAR *font,
                                      int ch);

  /********  Get the descent distance of a char.  ********/
  extern SWORD bfDECL bfGetChrDescent(bfFont bfFAR *font,
                                       int ch);

  /********  Get count of open fonts.  ********/
  extern SWORD bfDECL bfGetCount(void);

  /********  Set the font direction.  ********/
  extern bfState bfFAR * bfDECL bfSetDir(bfState bfFAR *state,
                                         bfDir dir);

  /********  Get the direction setting.  ********/
  extern bfDir bfDECL bfGetDir(bfState bfFAR *state);

  /********  Get the font directory.  ********/
  extern SBYTE bfFAR * bfDECL bfGetDirectory(void);

  /********  Set the font directory.  ********/
  extern SWORD bfDECL bfSetDirectory(char bfFAR *directory);

  /********  Get string text about the last error.  ********/
  extern SBYTE bfFAR * bfDECL bfGetErrorString(SWORD error);

  /********  Get the last error.  ********/
  extern SWORD bfDECL bfGetError(bfErrorClear errorclear);

  /********  Set the last error.  ********/
  extern void bfDECL bfSetError(SWORD err);

  /********  Get the face setting  ********/
  extern bfFace bfDECL bfGetFace(bfState bfFAR *state);

  /********  Set face modification.  ********/
  extern bfState bfFAR * bfDECL bfSetFace(bfState bfFAR *state,
                                          bfFace face);

  /********  Get the current font.  ********/
  extern bfFont bfFAR * bfDECL bfGetFont(bfState bfFAR *state);

  /********  Set the current font.  ********/
  extern bfState bfFAR * bfDECL bfSetFont(bfState bfFAR *state,
                                          bfFont bfFAR *font);

  /********  Get the ascent distance of a font.  ********/
  extern SWORD bfDECL bfGetFontAscent(bfFont bfFAR *font);

  /********  Get the maximum cell size for a font.  ********/
  extern bfFont bfFAR * bfDECL bfGetFontCellSize(bfFont bfFAR *font,
                                                 UWORD bfFAR *xs,
                                                 UWORD bfFAR *ys);

  /********  Get the descent distance of a font.  ********/
  extern SWORD bfDECL bfGetFontDescent(bfFont bfFAR *font);

  /********  Get the height of a font.  ********/
  extern SWORD bfDECL bfGetFontHeight(bfFont bfFAR *font);

  /********  Get justification settings.  ********/
  extern bfState bfFAR * bfDECL bfGetJust(bfState bfFAR *state,
                                          bfHorizJust bfFAR *hj,
                                          bfVertJust bfFAR *vj);

  /********  Set horiz and vert justification.  ********/
  extern bfState bfFAR * bfDECL bfSetJust(bfState bfFAR *state,
                                          bfHorizJust hj,
                                          bfVertJust vj);

  /********  Get font position.  ********/
  extern bfState bfFAR * bfDECL bfGetPos(bfState bfFAR *state,
                                         SWORD bfFAR *x,
                                         SWORD bfFAR *y);

  /********  Set the font position.  ********/
  extern bfState bfFAR * bfDECL bfSetPos(bfState bfFAR *state,
                                         SWORD x,
                                         SWORD y);

  /********  Get the spacing setting.  ********/
  extern SWORD bfDECL bfGetSpacing(bfState bfFAR *state);

  /********  Set character spacing.  ********/
  extern bfState bfFAR * bfDECL bfSetSpacing(bfState bfFAR *state,
                                             SWORD spacing);

  /********  Get the font state.  ********/
  extern bfState bfFAR * bfDECL bfGetState(void);

  /********  Set the font state.  ********/
  extern bfState bfFAR * bfDECL bfSetState(bfState bfFAR *state);

  /********  Get the width of the widest char in a string.  ********/
  extern SWORD bfDECL bfStrWidest(bfFont bfFAR *font,
                                  SBYTE bfFAR *istr);

  /********  Get the width of a string.  Char spacing in state!  ********/
  extern SWORD bfDECL bfStrWidth(bfState bfFAR *state,
                                 SBYTE bfFAR *istr);

  /********  Get the UL settings.  ********/
  extern bfState bfFAR * bfDECL bfGetUnderline(bfState bfFAR *state,
                                               SWORD bfFAR *uldist,
                                               SWORD bfFAR *ulthick);

  /********  Set underline settings.  ********/
  extern bfState bfFAR * bfDECL bfSetUnderline(bfState bfFAR *state,
                                               SWORD uldist,
                                               SWORD ulthick);


  /*  Platform interface functions.  */

  extern SWORD bfDECL bfGfxInit(void);

  extern void bfDECL bfGfxEnd(void);

  extern SWORD bfDECL bfGfxGetPalette(bfRGB bfFAR *palette,
                                      UWORD pstart,
                                      UWORD pcount);

  extern SWORD bfDECL bfGfxSetPalette(bfRGB bfFAR *palette,
                                      UWORD pstart,
                                      UWORD pcount);

// NOEXPORT here?  Make a cover func for export cause its used by underlines!
  extern void bfDECL bfGfxRect(bfDrawMode  dmode,
                               bfBackStyle style,
                               SWORD       stylesize,
                               bfColor     color,
                               SWORD       color256,
                               SWORD       x1,
                               SWORD       y1,
                               SWORD       x2,
                               SWORD       y2);

// NOEXPORT here?
  extern void bfDECL bfGfxLine(bfColor color,
                               SWORD   color256,
                               SWORD   x1,
                               SWORD   y1,
                               SWORD   x2,
                               SWORD   y2);

// NOEXPORT here?
  extern SWORD bfDECL bfGfxPack(bfColor bfFAR *pixin,
                                UBYTE   bfFAR *pixout,
                                SWORD         cnt,
                                bfRGB   bfFAR *palette,
                                SWORD         bytesperpix);

// NOEXPORT here?
  extern void bfDECL bfGfxPixel(bfColor color,
                                SWORD   color256,
                                SWORD   pixelx,
                                SWORD   pixely);

  extern void bfDECLNOEXPORT bfGfxGetCurrentBitmapSize(SWORD bfFAR *xsize,
                                                       SWORD bfFAR *ysize);

  extern void bfDECLNOEXPORT bfGfxSetColorsCallback(SWORD fore_or_drop);

  extern REAL bfDECLNOEXPORT bfGfxPow(REAL x, REAL y);

  extern SWORD bfDECLNOEXPORT bfGfxVirtualDraw(UBYTE bfFAR *AAbits,
                                               bfGFXType gfxtype,
                                               SWORD dropshadow,
                                               UWORD chrwidth,
                                               UWORD chrheight,
                                               SWORD x,
                                               SWORD y);



  /********  The internal string functions.  ********/
  extern SBYTE bfFAR * bfDECL bfMemCpy(SBYTE bfFAR *dst, SBYTE bfFAR *src,
                                       UWORD cnt);
  extern SBYTE bfFAR * bfDECL bfMemSet(SBYTE bfFAR *p, SWORD val, UWORD cnt);
  extern SBYTE bfFAR * bfDECL bfStrCpy(SBYTE bfFAR *dst,
                                       SBYTE bfFAR *src);
  extern SBYTE bfFAR * bfDECL bfStrCat(SBYTE bfFAR *str,
                                       SBYTE bfFAR *s2);
  extern SBYTE bfFAR * bfDECL bfStrChr(SBYTE bfFAR *str,
                                       SWORD chr);
  extern SWORD        bfDECL  bfStrICmp(SBYTE bfFAR *s1,
                                        SBYTE bfFAR *s2);
  extern SWORD        bfDECL  bfStrLen(SBYTE bfFAR *str);
  extern SBYTE        bfDECL  bfChrToUpper(SBYTE chr);



  /********  Here's the memory prototypes.  ********/
  extern void bfFAR * bfDECL bfHugeMalloc(ULONG size, bfMEMHANDLE bfFAR *hnd);
  extern void         bfDECL bfHugeFree(void bfFAR *m, bfMEMHANDLE bfFAR hnd);
  extern void bfFAR * bfDECL bfMalloc(ULONG size, bfMEMHANDLE bfFAR *handle);
  extern void         bfDECL bfFree(void bfFAR *m, bfMEMHANDLE bfFAR handle);



  #ifdef __cplusplus
    }
  #endif





#endif


